package org.implab.gradle.containers.cli;

import java.util.List;

public interface ComposeTraits {
    public final String UP_COMMAND = "up";
    public final String EXEC_COMMAND = "exec";
    public final String STOP_COMMAND = "stop";
    public final String RM_COMMAND = "rm";

    ProcessSpec compose(String command);

    default ProcessSpec stop() {
        return compose(STOP_COMMAND);
    }

    default ProcessSpec up(boolean detached) {
        var spec = compose(UP_COMMAND);
        if (detached)
            spec.args("--detach");
        return spec;
    }

    default ProcessSpec exec(String service, List<String> options,
            List<String> command) {
        return compose(EXEC_COMMAND).args(options)
                .args(service)
                .args(command);
    }

    default ProcessSpec rm(boolean removeVolumes) {
        var spec = compose(RM_COMMAND).args("--force", "--stop");
        if (removeVolumes)
            spec.args("--volumes");

        return spec;
    }
}
