package org.implab.gradle.containers.cli;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Optional;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

public class ImageName implements Serializable {

    private static final long serialVersionUID = -1990105923537254552L;

    final String authority;

    final String groupName;

    final String localName;

    final String tag;

    @JsonCreator(mode = JsonCreator.Mode.PROPERTIES)
    public ImageName(
            @JsonProperty("authority") String authority,
            @JsonProperty("groupName") String groupName,
            @JsonProperty("localName") String localName,
            @JsonProperty("tag") String tag) {
        if (localName == null || localName.isBlank())
            throw new IllegalArgumentException("localName can't be null or blank string");

        this.authority = authority;
        this.groupName = groupName;
        this.localName = localName;
        this.tag = tag;
    }

    public Optional<String> getAuthority() {
        return Optional.ofNullable(authority);
    }

    public ImageName withAuthority(String authority) {
        return new ImageName(authority, groupName, localName, tag);
    }

    public Optional<String> getGroupName() {
        return Optional.ofNullable(groupName);
    }

    public ImageName withGroupName(String groupName) {
        return new ImageName(authority, groupName, localName, tag);
    }

    public String getLocalName() {
        return localName;
    }

    public ImageName withLocalName(String localName) {
        if (localName == null || localName.isBlank())
            throw new IllegalArgumentException("localName can't be null or blank string");
        return new ImageName(authority, groupName, localName, tag);
    }

    public Optional<String> getTag() {
        return Optional.ofNullable(tag);
    }

    public ImageName withTag(String tag) {
        return new ImageName(authority, groupName, localName, tag);
    }

    @Override
    public String toString() {
        var path = new ArrayList<String>();

        getAuthority().ifPresent(path::add);
        getGroupName().ifPresent(path::add);
        path.add(getLocalName());

        var repo = String.join("/", path);

        return getTag().map(tag -> String.join(":", repo, tag)).orElse(repo);
    }
}
