package org.implab.gradle.containers.dsl;

import java.util.Optional;

import org.codehaus.groovy.runtime.DefaultGroovyMethods;
import org.gradle.api.plugins.ExtensionAware;
import org.gradle.api.plugins.ExtraPropertiesExtension;

public class ExtraProps {
    private final ExtraPropertiesExtension extraProps;

    protected ExtraProps(ExtraPropertiesExtension extraProps) {
        this.extraProps = extraProps;
    }

    public <T> Optional<T> get(String key, Class<T> clazz) {
        if (!extraProps.has(key))
            return Optional.empty();

        return Optional.ofNullable(DefaultGroovyMethods.asType(extraProps.get(key), clazz));
    }

    public <T> void put(String key, T value) {
        extraProps.set(key, value);
    }

    public void delete(String key) {
        extraProps.getProperties().remove(key);
    }

    public <T> MapEntry<T> prop(String key, Class<T> clazz) {
        return new MapEntry<>(extraProps.getProperties(), key, clazz);
    }

    public static Optional<ExtraProps> extra(Object target) {
        return target instanceof ExtensionAware
                ? Optional.of(new ExtraProps(
                        ((ExtensionAware) target).getExtensions().getExtraProperties()))
                : Optional.empty();

    }
}
