package org.implab.gradle.containers.tasks;

import java.util.Optional;

import org.gradle.api.provider.ListProperty;
import org.gradle.api.tasks.Internal;
import org.implab.gradle.containers.cli.RedirectFrom;
import org.implab.gradle.containers.cli.RedirectTo;
import org.implab.gradle.containers.dsl.OptionsMixin;
import org.implab.gradle.containers.dsl.RedirectFromSpec;
import org.implab.gradle.containers.dsl.RedirectToSpec;

public abstract class BaseExecTask extends DockerCliTask implements OptionsMixin {
    private final RedirectToSpec redirectStderr = new RedirectToSpec();

    private final RedirectToSpec redirectStdout = new RedirectToSpec();

    private final RedirectFromSpec redirectStdin = new RedirectFromSpec();

    private boolean detached = false;

    private boolean interactive = false;

    private boolean allocateTty = false;

    @Internal
    public abstract ListProperty<String> getCommandLine();


    /**
     * STDIN redirection, if not specified, no input will be passed to the command
     */
    @Internal
    public RedirectFromSpec getStdin() {
        return redirectStdin;
    }

    /**
     * STDOUT redirection, if not specified, redirected to logger::info
     */
    @Internal
    public RedirectToSpec getStdout() {
        return redirectStdout;
    }

    /**
     * STDERR redirection, if not specified, redirected to logger::error
     */
    @Internal
    public RedirectToSpec getStderr() {
        return redirectStderr;
    }

    /**
     * Specified whether the container should run in background. If the container
     * has explicit IO redirects it will run in the foreground.
     */
    @Internal
    public boolean isDetached() {
        // if IO was redirected the container should run in foreground
        if (redirectStdin.isRedirected() || redirectStdout.isRedirected() || redirectStderr.isRedirected())
            return false;

        return detached;
    }

    public void setDetached(boolean value) {
        detached = value;
    }

    @Internal
    public boolean getAllocateTty() {
        return allocateTty;
    }

    public void setAllocateTty(boolean value) {
        allocateTty = value;
    }

    /**
     * Specified the interactive flag for the container, if the task specified
     * STDIN redirection this flag is enabled automatically.
     */
    @Internal
    public boolean isInteractive() {
        // enable interactive mode when processing standard input
        return redirectStdin.isRedirected() || interactive;
    }

    /** Appends specified parameters to the command line */
    void commandLine(String... args) {
        getCommandLine().addAll(args);
    }


    @Override
    protected Optional<RedirectTo> stdoutRedirection() {
        return redirectStdout.getRedirection().or(super::stdoutRedirection);
    }

    @Override
    protected Optional<RedirectTo> stderrRedirection() {
        return redirectStderr.getRedirection().or(super::stderrRedirection);
    }

    @Override
    protected Optional<RedirectFrom> stdinRedirection() {
        return redirectStdin.getRedirection();
    }

}
