package org.implab.gradle.containers.tasks;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutionException;

import org.gradle.api.provider.Property;
import org.gradle.api.tasks.Internal;
import org.gradle.api.tasks.TaskAction;

public abstract class StopContainer extends DockerCliTask implements ContainerTaskMixin {

    @Internal
    public abstract Property<String> getStopSignal();

    @Internal
    public abstract Property<Integer> getStopTimeout();

    public StopContainer() {
        onlyIfReason("Container doesn't exists", self -> getContainerId().map(this::containerExists).getOrElse(false));
    }

    @TaskAction
    public void run() throws InterruptedException, IOException, ExecutionException {
        var options = new ArrayList<String>();

        if (getStopSignal().isPresent())
            options.addAll(List.of("--signal", getStopSignal().get()));

        if (getStopTimeout().isPresent())
            options.addAll(List.of("--time", getStopTimeout().map(Object::toString).get()));

        exec(docker().containerStop(getContainerId().get(), options));
    }

}
