package org.implab.gradle.containers.tasks;

import java.io.IOException;
import java.util.Set;
import java.util.concurrent.ExecutionException;

import org.gradle.api.provider.Property;
import org.gradle.api.provider.SetProperty;
import org.gradle.api.tasks.Input;
import org.gradle.api.tasks.TaskAction;
import org.implab.gradle.containers.cli.Utils;

public abstract class TagImage extends DockerCliTask {
    @Input
    public abstract Property<Object> getSrcImage();

    @Input
    public abstract SetProperty<Object> getTags();

    @Input
    @Deprecated
    public abstract Property<Object> getDestImage();

    private Set<String> getImageTags() {
        var tags = getTags().map(Utils::mapToString).get();
        tags.add(getDestImage().map(Object::toString).get());
        return tags;
    }

    public TagImage() {
        this.setOnlyIf(self -> getImageTags().size() > 0);
    }

    @TaskAction
    public void run() throws InterruptedException, IOException, ExecutionException {
        var tags = getImageTags();
        var src = getSrcImage().map(Object::toString).get();

        for (var tag : tags) {
            getLogger().info("Tag: {}", tag);
            exec(docker().imageTag(src, tag));
        }
    }
}
